class RiotAPIException(Exception):
    def __init__(self, message: str, error_code: int):
        super().__init__(message)
        self.message = message
        self.error_code = error_code

    def __str__(self) -> str:
        return f"{self.message} (Error Code: {self.error_code})"


class BadRequest(RiotAPIException):
    pass


class Unauthorized(RiotAPIException):
    pass


class Forbidden(RiotAPIException):
    pass


class DataNotFound(RiotAPIException):
    pass


class MethodNotAllowed(RiotAPIException):
    pass


class UnsupportedMediaType(RiotAPIException):
    pass


class RateLimitExceeded(RiotAPIException):
    pass


class InternalServerError(RiotAPIException):
    pass


class BadGateway(RiotAPIException):
    pass


class ServiceUnavailable(RiotAPIException):
    pass


class GatewayTimeout(RiotAPIException):
    pass


class UnknownError(RiotAPIException):
    pass


class RequestTimeout(RiotAPIException):
    pass



from typing import Any

from pyke import Region

from .._base_client import _BaseApiClient


class StatusEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def platform_data(self, region: Region) -> dict[Any, Any]:
        """# Get League of Legends status for the given platform

        **Example:**  
            `status = await api.lol_status.platform_data(Region.EUW)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = "/lol/status/v4/platform-data"
        data = await self._client._region_request(region=region, path=path)

        return data



"""
Data Dragon module.

This module provides access to League of Legends static data through Data Dragon.
It includes classes and functions for retrieving champion information, items, runes, maps,
and other game assets.
"""



"""# pyke

[![PyPI - Version](https://img.shields.io/pypi/v/pyke-lol)](https://pypi.org/project/pyke-lol/)
[![Documentation](https://img.shields.io/badge/Documentation-blue)](https://diodemusic.github.io/pyke/)

**pyke** is a thin async Riot API wrapper for League of Legends.

## Installation

Install the latest version directly from PyPI:

```bash
pip install pyke-lol
```

> You need Python 3.10+

---

## Quickstart

```py
import asyncio

from pyke import Continent, Pyke, exceptions

async def main() -> None:
    # Initialize the API
    async with Pyke("RGAPI-...", timeout=60, print_url=True, print_rate_limit=True) as api:
        # Every pyke method follows the same convention as the Riot API
        # For example account/v1/accounts/by-riot-id/{gameName}/{tagLine} becomes:
        account = await api.account.by_riot_id(Continent.EUROPE, "saves", "000")

        print(f"Riot ID: {account['gameName']}#{account['tagLine']}")
        print(f"PUUID:   {account['puuid']}")

        # pyke throws typed exceptions matching Riot API error codes
        try:
            region = await api.account.region_by_puuid(
                Continent.EUROPE, account["puuid"]
            )
        except exceptions.DataNotFound as e:
            print(e)  # Output: Data not found (Error Code: 404)
            quit()

        print(f"Region:  {region['region']}")
```

## Custom Exception Handling

Typed exceptions for all HTTP status codes:

```py
from pyke import exceptions

try:
    summoner = await api.summoner.by_puuid(Region.EUW, "NonExistentPuuid")
except exceptions.DataNotFound as e:
    print(f"Not found: {e}")     # Data not found (Error Code: 404)
except exceptions.RateLimitExceeded as e:
    print(f"Rate limited: {e}")  # Rate limit exceeded (Error Code: 429)
except exceptions.InternalServerError as e:
    print(f"Server error: {e}")  # Internal server error (Error Code: 500)
```

## Resources

- **[API Documentation](https://diodemusic.github.io/pyke/pyke.html)**
- **[Examples Directory](https://github.com/diodemusic/pyke/tree/master/examples)**
- **[PyPI Package](https://pypi.org/project/pyke-lol/)**
- **[GitHub Repository](https://github.com/diodemusic/pyke)**

---

For any questions or help, please reach out on Discord: `.irm`
"""

from . import ddragon, endpoints, enums, exceptions
from .__version__ import __author__, __title__, __version__
from .enums.continent import Continent
from .enums.division import Division
from .enums.level import Level
from .enums.queue import Queue
from .enums.region import Region
from .enums.tier import Tier
from .enums.type import Type
from .main import DataDragon, Pyke

__all__ = [
    "ddragon",
    "exceptions",
    "Continent",
    "Division",
    "Level",
    "Queue",
    "Region",
    "Tier",
    "Type",
    "DataDragon",
    "Pyke",
    "__author__",
    "__title__",
    "__version__",
    "endpoints",
    "enums",
]



"""
The **enums** submodule contains enumerations that represent constants used throughout the Riot API wrapper.

These enums define values for regions, queue types, tiers, divisions, continents, and other fixed API parameters, ensuring consistent and type-safe usage across the package.

**Typical usage:**

    from pyke import Continent, Region, Tier

    print(Continent.EUROPE)
    print(Region.EUW)
    print(Tier.PLATINUM)
"""



from typing import Any

from pyke import Continent

from .._base_client import _BaseApiClient


class AccountEndpoint:
    """There are three routing values for account-v1; americas, asia, and europe. You can query for any account in any region. We recommend using the nearest cluster."""

    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def by_puuid(self, continent: Continent, puuid: str) -> dict[Any, Any]:
        """# Get account by puuid

        **Example:**  
            `account = await api.account.by_puuid(Continent.EUROPE, "some puuid")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/riot/account/v1/accounts/by-puuid/{puuid}"
        data = await self._client._continent_request(continent=continent, path=path)

        return data

    async def by_riot_id(
        self, continent: Continent, game_name: str, tag_line: str
    ) -> dict[Any, Any]:
        """# Get account by riot id

        **Example:**  
            `account = await api.account.by_riot_id(Continent.EUROPE, "saves", "000")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `game_name (str)` Riot id game name.  
            `tag_line (str)` Riot id tag line.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/riot/account/v1/accounts/by-riot-id/{game_name}/{tag_line}"
        data = await self._client._continent_request(continent=continent, path=path)

        return data

    async def region_by_puuid(self, continent: Continent, puuid: str) -> dict[Any, Any]:
        """# Get active region (lol and tft)

        **Example:**  
            `region = await api.account.region_by_puuid(Continent.EUROPE, "some puuid")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/riot/account/v1/region/by-game/lol/by-puuid/{puuid}"
        data = await self._client._continent_request(continent=continent, path=path)

        return data



from typing import Any

from pyke import Level, Region

from .._base_client import _BaseApiClient


class ChallengesEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def config(self, region: Region) -> list[dict[Any, Any]]:
        """# List of all basic challenge configuration information (includes all translations for names and descriptions)

        **Example:**  
            `configs = await api.lol_challenges.config(Region.EUW)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = "/lol/challenges/v1/challenges/config"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def percentiles(self, region: Region) -> dict[Any, Any]:
        """# Map of level to percentile of players who have achieved it - keys: ChallengeId -> Season -> Level -> percentile of players who achieved it

        **Example:**  
            `percentiles = await api.lol_challenges.percentiles(Region.EUW)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = "/lol/challenges/v1/challenges/percentiles"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def config_by_challenge_id(
        self, region: Region, challenge_id: int
    ) -> dict[Any, Any]:
        """# Get challenge configuration (REST)

        **Example:**  
            `config = await api.lol_challenges.config_by_challenge_id(Region.EUW, 123456)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `challenge_id (int)` Challenge id integer.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/challenges/v1/challenges/{challenge_id}/config"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def leaderboards_by_level(
        self, region: Region, level: Level, challenge_id: int
    ) -> list[dict[Any, Any]]:
        """# Return top players for each level. Level must be MASTER, GRANDMASTER or CHALLENGER

        **Example:**  
            `players = await api.lol_challenges.leaderboards_by_level(Region.EUW, Level.MASTER, 123456)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `level (Level)` Challenge [Level](/pyke/pyke.html#Level).  
            `challenge_id (int)` Challenge id integer.  

        **Returns:**  
            `list[dict[Any, Any]]` data
        """  # fmt: skip

        path = f"/lol/challenges/v1/challenges/{challenge_id}/leaderboards/by-level/{level.value}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def percentiles_by_challenge_id(
        self, region: Region, challenge_id: int
    ) -> dict[Level, int]:
        """# Dictionary of level to percentile of players who have achieved it

        **Example:**  
            `percentiles = await api.lol_challenges.percentiles_by_challenge_id(Region.EUW, 123456)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `challenge_id (int)` Challenge id integer.  

        **Returns:**  
            `dict[Level, int]` Python dictionary {[Level](/pyke/pyke.html#Level): percentile of players who achieved the challenge}
        """  # fmt: skip

        path = f"/lol/challenges/v1/challenges/{challenge_id}/percentiles"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def by_puuid(self, region: Region, puuid: str) -> dict[Any, Any]:
        """# Returns player information with list of all progressed challenges (REST)

        **Example:**  
            `player_info = await api.lol_challenges.by_puuid(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/challenges/v1/player-data/{puuid}"
        data = await self._client._region_request(region=region, path=path)

        return data



from __future__ import annotations

import json
from typing import Any

import httpx
from httpx import Response

from . import exceptions
from .enums.continent import Continent
from .enums.region import Region


class _BaseApiClient:  # pyright: ignore[reportUnusedClass]
    CONTINENT_BASE = "https://{continent}.api.riotgames.com"
    REGION_BASE = "https://{region}.api.riotgames.com"

    def __init__(
        self, api_key: str | None, timeout: int, print_url: bool, print_rate_limit: bool
    ) -> None:
        if api_key is None:
            raise ValueError("API key is required, please pass a valid Riot API key.")
        if not api_key.startswith("RGAPI-") or len(api_key) != 42:
            raise ValueError("Bad API key, please pass a valid Riot API key.")

        self.api_key = api_key
        self.timeout = timeout
        self.print_url = print_url
        self.print_rate_limit = print_rate_limit
        self.client = httpx.AsyncClient(timeout=httpx.Timeout(self.timeout))
        self._status_code_registry = {
            400: exceptions.BadRequest("Bad request", 400),
            401: exceptions.Unauthorized("Unauthorized", 401),
            403: exceptions.Forbidden("Forbidden", 403),
            404: exceptions.DataNotFound("Data not found", 404),
            405: exceptions.MethodNotAllowed("Method not allowed", 405),
            415: exceptions.UnsupportedMediaType("Unsupported media type", 415),
            500: exceptions.InternalServerError("Internal server error", 500),
            502: exceptions.BadGateway("Bad gateway", 502),
            503: exceptions.ServiceUnavailable("Service unavailable", 503),
            504: exceptions.GatewayTimeout("Gateway timeout", 504),
        }

    def _get_count(self, response: Response) -> int:
        header_value = response.headers.get("X-App-Rate-Limit-Count")

        if not header_value:
            return 0

        parts = header_value.split(":")

        if len(parts) < 1 or not parts[0]:
            return 0

        try:
            return int(parts[0])
        except ValueError:
            return 0

    def _get_limit(self, response: Response) -> int:
        header_value = response.headers.get("X-App-Rate-Limit")

        if not header_value:
            return 100

        parts = header_value.split(":")

        if len(parts) < 1 or not parts[0]:
            return 100

        try:
            return int(parts[0])
        except ValueError:
            return 100

    def _response_json(self, response: Response) -> Any:
        try:
            return response.json()
        except json.JSONDecodeError:
            raise exceptions.InternalServerError("Could not decode JSON", 500)
        except ValueError:
            raise exceptions.InternalServerError("Empty JSON response", 500)

    async def _get(self, url: str, params: dict[Any, Any] | None = None) -> Any:
        if self.print_url:
            print(f"URL:        {url}")

        headers = {"X-Riot-Token": self.api_key}

        try:
            response = await self.client.get(url, headers=headers, params=params)
        except httpx.TimeoutException:
            raise exceptions.RequestTimeout(
                f"Request timed out after {self.timeout} seconds", 408
            )
        except httpx.RequestError as exc:
            raise exceptions.RequestTimeout(str(exc), 408)

        if self.print_rate_limit:
            self.count = self._get_count(response)
            self.limit = self._get_limit(response)
            print(f"Rate limit: ({self.count}/{self.limit})")

        code = response.status_code

        if code == 200:
            return self._response_json(response)
        elif code == 429:
            raise exceptions.RateLimitExceeded("Rate limit exceeded", code)

        raise self._status_code_registry.get(
            code,
            exceptions.UnknownError(
                "Unexpected response, something has gone terribly wrong", code
            ),
        )

    async def _continent_request(
        self, continent: Continent, path: str, params: dict[Any, Any] | None = None
    ) -> Any:
        url = f"{self.CONTINENT_BASE.format(continent=continent.value)}{path}"

        return await self._get(url, params)

    async def _region_request(
        self, region: Region, path: str, params: dict[Any, Any] | None = None
    ) -> Any:
        url = f"{self.REGION_BASE.format(region=region.value)}{path}"

        return await self._get(url, params)

    async def aclose(self) -> None:
        await self.client.aclose()



from typing import Any

from pyke import Region

from .._base_client import _BaseApiClient


class ChampionEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def rotations(self, region: Region) -> dict[Any, Any]:
        """# Returns champion rotations, including free-to-play and low-level free-to-play rotations

        **Example:**  
            `rotations = await api.champion.rotations(Region.EUW)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = "/lol/platform/v3/champion-rotations"
        data = await self._client._region_request(region=region, path=path)

        return data



from typing import Any

from .._base_data_dragon_client import _BaseDataDragonClient


class ProfileiconData:
    def __init__(self, client: _BaseDataDragonClient):
        self._client = client

    def get_all(self, locale: str) -> dict[str, Any]:
        """# Get all profileicon by locale

        **Example:**
            `profileicon = ddragon.profileicon.get_all("en_GB")`

        **Args:**
            `locale (str)` Locale to use.

        **Returns:**
            `dict[str, any]`
        """  # fmt: skip

        return self._client._data_dragon_cdn_request(locale, "profileicon")



from __future__ import annotations

import json
from typing import Any

import httpx
from httpx import Response

from . import exceptions


class _BaseDataDragonClient:  # pyright: ignore[reportUnusedClass]
    DATA_DRAGON_BASE = "https://ddragon.leagueoflegends.com"

    def __init__(self, version: str | None, timeout: int) -> None:
        self.timeout = timeout
        self.client = httpx.Client(timeout=self.timeout)

        if version is None:
            self.version = self._get_latest_version()
        else:
            self.version = version

        self._status_code_registry = {
            400: exceptions.BadRequest("Bad request", 400),
            401: exceptions.Unauthorized("Unauthorized", 401),
            403: exceptions.Forbidden("Forbidden", 403),
            404: exceptions.DataNotFound("Data not found", 404),
            405: exceptions.MethodNotAllowed("Method not allowed", 405),
            415: exceptions.UnsupportedMediaType("Unsupported media type", 415),
            500: exceptions.InternalServerError("Internal server error", 500),
            502: exceptions.BadGateway("Bad gateway", 502),
            503: exceptions.ServiceUnavailable("Service unavailable", 503),
            504: exceptions.GatewayTimeout("Gateway timeout", 504),
        }

    def _get_latest_version(self) -> str:
        versions = self._data_dragon_request("/api/versions.json")

        try:
            version = versions[0]
        except IndexError:
            raise exceptions.InternalServerError(
                "Could not index version from ddragon", 500
            )

        return version

    def _response_json(self, response: Response) -> Any:
        try:
            return response.json()
        except json.JSONDecodeError:
            raise exceptions.InternalServerError("Could not decode JSON", 500)
        except ValueError:
            raise exceptions.InternalServerError("Empty JSON response", 500)

    def _get(self, url: str) -> Any:
        try:
            response = self.client.get(url)
        except httpx.TimeoutException:
            raise exceptions.RequestTimeout(
                f"Request timed out after {self.timeout} seconds", 408
            )

        code = response.status_code

        if code == 200:
            return self._response_json(response)
        else:
            raise self._status_code_registry.get(
                code,
                exceptions.UnknownError(
                    "Unexpected response, something has gone terribly wrong", code
                ),
            )

    def _data_dragon_request(self, path: str) -> Any:
        url = f"{self.DATA_DRAGON_BASE}{path}"

        return self._get(url)

    def _data_dragon_cdn_request(self, locale: str, endpoint: str) -> Any:
        url = (
            f"{self.DATA_DRAGON_BASE}/cdn/{self.version}/data/{locale}/{endpoint}.json"
        )

        return self._get(url)



# pyke

from __future__ import annotations

from ._base_client import _BaseApiClient
from ._base_data_dragon_client import _BaseDataDragonClient
from .ddragon.challenges import ChallengesData
from .ddragon.champion import ChampionData
from .ddragon.champion_full import ChampionFullData
from .ddragon.item import ItemData
from .ddragon.itemmodifiers import ItemmodifiersData
from .ddragon.language import LanguageData
from .ddragon.map import MapData
from .ddragon.missionassets import MissionassetsData
from .ddragon.profileicon import ProfileiconData
from .ddragon.runes_reforged import RunesReforgedData
from .ddragon.spellbuffs import SpellbuffsData
from .ddragon.sticker import StickerData
from .ddragon.summoner import SummonerData
from .endpoints.account import AccountEndpoint
from .endpoints.champion import ChampionEndpoint
from .endpoints.champion_mastery import ChampionMasteryEndpoint
from .endpoints.clash import ClashEndpoint
from .endpoints.league import LeagueEndpoint
from .endpoints.league_exp import LeagueExpEndpoint
from .endpoints.lol_challenges import ChallengesEndpoint
from .endpoints.lol_status import StatusEndpoint
from .endpoints.match import MatchEndpoint
from .endpoints.spectator import SpectatorEndpoint
from .endpoints.summoner import SummonerEndpoint


class Pyke:
    """# Main entrypoint for interacting with the Riot API

    **Example:**  
        `api = Pyke("RGAPI-...")`

    **Args:**  
        `api_key (str | None)` Your Riot API key.  
        `timeout (int, optional)` Request timeout in seconds. Defaults to 60.  
        `print_url (bool, optional)` Print endpoint URL.
        `print_rate_limit (bool, optional)` Print rate limit usage.
    """  # fmt: skip

    def __init__(
        self,
        api_key: str | None,
        timeout: int = 60,
        print_url: bool = True,
        print_rate_limit: bool = True,
    ) -> None:
        self._client = _BaseApiClient(api_key, timeout, print_url, print_rate_limit)

        self.account = AccountEndpoint(self._client)
        self.champion_mastery = ChampionMasteryEndpoint(self._client)
        self.champion = ChampionEndpoint(self._client)
        self.clash = ClashEndpoint(self._client)
        self.league_exp = LeagueExpEndpoint(self._client)
        self.league = LeagueEndpoint(self._client)
        self.lol_challenges = ChallengesEndpoint(self._client)
        self.lol_status = StatusEndpoint(self._client)
        self.match = MatchEndpoint(self._client)
        self.spectator = SpectatorEndpoint(self._client)
        self.summoner = SummonerEndpoint(self._client)

    async def __aenter__(self) -> Pyke:
        return self

    async def __aexit__(self, exc_type, exc, tb):  # type: ignore
        await self.aclose()

    async def aclose(self):
        await self._client.aclose()


class DataDragon:
    def __init__(self, version: str | None = None, timeout: int = 10) -> None:
        self._client = _BaseDataDragonClient(version, timeout)

        # self.versions = VersionsData(self._client)
        self.spellbuffs = SpellbuffsData(self._client)
        self.item = ItemData(self._client)
        self.runes_reforged = RunesReforgedData(self._client)
        self.language = LanguageData(self._client)
        self.itemmodifiers = ItemmodifiersData(self._client)
        self.champion_full = ChampionFullData(self._client)
        self.summoner = SummonerData(self._client)
        self.champion = ChampionData(self._client)
        self.challenges = ChallengesData(self._client)
        self.missionassets = MissionassetsData(self._client)
        self.sticker = StickerData(self._client)
        self.profileicon = ProfileiconData(self._client)
        self.map = MapData(self._client)



from __future__ import annotations

from typing import Any

from pyke import Region

from .._base_client import _BaseApiClient


class ChampionMasteryEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def masteries_by_puuid(
        self, region: Region, puuid: str
    ) -> list[dict[Any, Any]]:
        """# Get all champion mastery entries sorted by number of champion points descending

        **Example:**  
            `masteries = await api.champion_mastery.masteries_by_puuid(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = f"/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def by_puuid_and_champion_id(
        self, region: Region, puuid: str, champion_id: int
    ) -> dict[Any, Any]:
        """# Get a champion mastery by puuid and champion ID

        **Example:**  
            `mastery = await api.champion_mastery.by_puuid_and_champion_id(Region.EUW, "some puuid", 29)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  
            `champion_id (int)` Champion ID for this entry.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/by-champion/{champion_id}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def masteries_by_puuid_top(
        self, region: Region, puuid: str, count: int | None = None
    ) -> list[dict[Any, Any]]:
        """# Get specified number of top champion mastery entries sorted by number of champion points descending

        **Example:**  
            `masteries = await api.champion_mastery.masteries_by_puuid_top(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  
            `count (int | None, optional)` Number of entries to retrieve. defaults to 3.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = f"/lol/champion-mastery/v4/champion-masteries/by-puuid/{puuid}/top"
        params = {"count": count}
        data = await self._client._region_request(
            region=region, path=path, params=params
        )

        return data

    async def score_by_puuid(self, region: Region, puuid: str) -> int:
        """# Get a player's total champion mastery score, which is the sum of individual champion mastery levels

        **Example:**  
            `score = await api.champion_mastery.score_by_puuid(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `int` Player's total champion mastery score.
        """  # fmt: skip

        path = f"/lol/champion-mastery/v4/scores/by-puuid/{puuid}"
        data = await self._client._region_request(region=region, path=path)

        return int(data)



from typing import Any

from pyke import Region

from .._base_client import _BaseApiClient


class SummonerEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def by_puuid(self, region: Region, puuid: str) -> dict[Any, Any]:
        """Get a summoner by PUUID.

        **Example:**  
            `api.summoner.by_puuid(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/summoner/v4/summoners/by-puuid/{puuid}"
        data = await self._client._region_request(region=region, path=path)

        return data



from typing import Any

from pyke import Region

from .._base_client import _BaseApiClient


class ClashEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def by_puuid(self, region: Region, puuid: str) -> list[dict[Any, Any]]:
        """# Get players by puuid

        **Example:**  
            `players = await api.clash.by_puuid(Region.EUW, "some puuid")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = f"/lol/clash/v1/players/by-puuid/{puuid}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def by_team_id(self, region: Region, team_id: str) -> dict[Any, Any]:
        """# Get team by ID

        **Example:**  
            `team = await api.clash.by_team_id(Region.EUW, "some team id")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `team_id (str)` Team id of the clash team.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/clash/v1/teams/{team_id}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def tournaments(self, region: Region) -> list[dict[Any, Any]]:
        """# Get all active or upcoming tournaments

        **Example:**  
            `tournaments = await api.clash.tournaments(Region.EUW)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = "/lol/clash/v1/tournaments"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def tournament_by_team_id(
        self, region: Region, team_id: str
    ) -> dict[Any, Any]:
        """# Get tournament by team ID

        **Example:**  
            `tournament = await api.clash.tournament_by_team_id(Region.EUW, "some team id")`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `team_id (str)` Team id of the clash team.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/clash/v1/tournaments/by-team/{team_id}"
        data = await self._client._region_request(region=region, path=path)

        return data

    async def tournament_by_tournament_id(
        self, region: Region, tournament_id: int
    ) -> dict[Any, Any]:
        """# Get tournament by ID

        **Example:**  
            `tournament = await api.clash.tournament_by_tournament_id(Region.EUW, 12345)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `tournament_id (int)` Tournament id of the clash.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/clash/v1/tournaments/{tournament_id}"
        data = await self._client._region_request(region=region, path=path)

        return data



from __future__ import annotations

from typing import Any

from pyke import Division, Queue, Region, Tier

from .._base_client import _BaseApiClient


class LeagueExpEndpoint:
    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def by_queue_tier_division(
        self,
        region: Region,
        queue: Queue,
        tier: Tier,
        division: Division,
        page: int | None = None,
    ) -> list[dict[Any, Any]]:
        """# Get all the league entries

        **Example:**  
        `entries = await api.league_exp.by_queue_tier_division(Region.EUW, Queue.SOLO_DUO, Tier.GOLD, Division.II)`

        **Args:**  
            `region (Region)` [Region](/pyke/pyke.html#Region) to execute against.  
            `queue (Queue)` Ranked [Queue](/pyke/pyke.html#Queue) type.  
            `tier (Tier)` Ranked [Tier](/pyke/pyke.html#Tier).  
            `division (Division)` Ranked [Division](/pyke/pyke.html#Division).  
            `page (int, optional)` Starts with page 1. Defaults to 1.  

        **Returns:**  
            `list[dict[Any, Any]]`
        """  # fmt: skip

        path = f"/lol/league-exp/v4/entries/{queue.value}/{tier.value}/{division.value}"
        params = {"page": page}
        data = await self._client._region_request(
            region=region, path=path, params=params
        )

        return data



from __future__ import annotations

from typing import Any

from pyke import Continent, Type

from .. import exceptions
from .._base_client import _BaseApiClient


class MatchEndpoint:
    """The AMERICAS routing value serves NA, BR, LAN and LAS. The ASIA routing value serves KR and JP. The EUROPE routing value serves EUNE, EUW, ME1, TR and RU. The SEA routing value serves OCE, SG2, TW2 and VN2."""

    def __init__(self, client: _BaseApiClient):
        self._client = client

    async def match_ids_by_puuid(
        self,
        continent: Continent,
        puuid: str,
        start_time: int | None = None,
        end_time: int | None = None,
        queue: int | None = None,
        type: Type | None = None,
        start: int | None = None,
        count: int | None = None,
    ) -> list[str]:
        """# Get a list of match ids by puuid

        **Example:**  
            `match_ids = await api.match.match_ids_by_puuid(Continent.EUROPE, "some puuid")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `puuid (str)` Encrypted PUUID. Exact length of 78 characters.  
            `start_time (int | None, optional)` Epoch timestamp in seconds. The matchlist started storing timestamps on June 16th, 2021. Any matches played before June 16th, 2021 won't be included in the results if the startTime filter is set. Defaults to None.  
            `end_time (int | None, optional)` Epoch timestamp in seconds. Defaults to None.  
            `queue (int | None, optional)` Filter the list of match ids by a specific queue id. This filter is mutually inclusive of the type filter meaning any match ids returned must match both the queue and type filters. Defaults to None.  
            `type (Type | None, optional)` Filter the list of match ids by the [Type](/pyke/pyke.html#Type) of match. This filter is mutually inclusive of the queue filter meaning any match ids returned must match both the queue and type filters. Defaults to None.  
            `start (int | None, optional)` Start index. Defaults to 0.  
            `count (int | None, optional)` Valid values: 0 to 100. Number of match ids to return. Defaults to 20.  

        **Returns:**  
            `list[str]` List of match id strings.
        """  # fmt: skip

        path = f"/lol/match/v5/matches/by-puuid/{puuid}/ids"

        if type:
            type_str = type.value
        else:
            type_str = None

        params = {
            "startTime": start_time,
            "endTime": end_time,
            "queue": queue,
            "type": type_str,
            "start": start,
            "count": count,
        }
        params = {k: v for k, v in params.items() if v is not None}
        data = await self._client._continent_request(
            continent=continent, path=path, params=params
        )

        if not isinstance(data, list):
            raise exceptions.DataNotFound(
                message="Expected list of match IDs but received invalid response format",
                error_code=404,
            )

        if not all(
            isinstance(item, str)
            for item in data  # pyright: ignore[reportUnknownVariableType]
        ):
            raise exceptions.DataNotFound(
                message="Match ID list contains non-string values", error_code=404
            )

        return data  # pyright: ignore[reportUnknownVariableType]

    async def by_match_id(self, continent: Continent, match_id: str) -> dict[Any, Any]:
        """# Get a match by match id

        **Example:**  
            `match = await api.match.by_match_id(Continent.EUROPE, "EUW1_1234567890")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `match_id (str)` Match id string.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/match/v5/matches/{match_id}"
        data = await self._client._continent_request(continent=continent, path=path)

        return data

    async def timeline_by_match_id(
        self, continent: Continent, match_id: str
    ) -> dict[Any, Any]:
        """# Get a match timeline by match id

        **Example:**  
            `timeline = await api.match.timeline_by_match_id(Continent.EUROPE, "EUW1_1234567890")`

        **Args:**  
            `continent (Continent)` [Continent](/pyke/pyke.html#Continent) to execute against.  
            `match_id (str)` Match id string.  

        **Returns:**  
            `dict[Any, Any]`
        """  # fmt: skip

        path = f"/lol/match/v5/matches/{match_id}/timeline"
        data = await self._client._continent_request(continent=continent, path=path)

        return data
